package com.qmth.themis.business.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.enums.Platform;
import com.qmth.themis.common.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;

/**
 * @Description: jwt工具类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/4/8
 */
public class JwtUtil {
    private final static Logger log = LoggerFactory.getLogger(JwtUtil.class);

    /**
     * 校验token是否正确
     *
     * @param token
     * @param userId
     * @return
     */
    public static boolean verify(String token, Long userId) {
        try {
            //根据密码生成JWT效验器
            String secret = getClaim(token, SystemConstant.JWT_USERID) + SystemConstant.JWT_SECRET;
            Algorithm algorithm = Algorithm.HMAC256(secret);
            JWTVerifier verifier = JWT.require(algorithm)
                    .withClaim(SystemConstant.JWT_USERID, String.valueOf(userId))
                    .build();
            //效验TOKEN
            verifier.verify(token);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException(ExceptionResultEnum.TOKEN_NO);
        }
    }

    /**
     * 校验token是否正确
     *
     * @param token
     * @param userId
     * @param platform
     * @param deviceId
     * @return
     */
    public static boolean verify(String token, Long userId, Platform platform, String deviceId) {
        try {
            //根据密码生成JWT效验器
            String secret = getClaim(token, SystemConstant.JWT_USERID) + SystemConstant.JWT_SECRET;
            Algorithm algorithm = Algorithm.HMAC256(secret);
            JWTVerifier verifier = JWT.require(algorithm)
                    .withClaim(SystemConstant.JWT_USERID, String.valueOf(userId))
                    .withClaim(SystemConstant.PLATFORM, platform.name())
                    .withClaim(SystemConstant.DEVICE_ID, deviceId)
                    .build();
            //效验TOKEN
            verifier.verify(token);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            throw new BusinessException(ExceptionResultEnum.TOKEN_NO);
        }
    }

    /**
     * 获得token中的信息无需secret解密也能获得
     *
     * @param token
     * @param key
     * @return
     */
    public static String getClaim(String token, String key) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getClaim(key).asString();
        } catch (JWTDecodeException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 生成签名
     *
     * @param userId
     * @return
     */
    public static String sign(Long userId) {
        try {
            String secret = userId + SystemConstant.JWT_SECRET;
            Long start = System.currentTimeMillis();
            Date date = new Date(start + SystemConstant.JWT_EXPIRE_TIME);
            Algorithm algorithm = Algorithm.HMAC256(secret);
            // 附带username信息
            return JWT.create()
                    .withClaim(SystemConstant.JWT_USERID, String.valueOf(userId))
                    .withClaim(SystemConstant.JWT_CURRENT_TIME, String.valueOf(start))
                    .withExpiresAt(date)
                    .sign(algorithm);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 生成签名
     *
     * @param userId
     * @param platform
     * @param deviceId
     * @return
     */
    public static String sign(Long userId, Platform platform, String deviceId) {
        try {
            String secret = userId + SystemConstant.JWT_SECRET;
            Long start = System.currentTimeMillis();
            Date date = new Date(start + SystemConstant.JWT_EXPIRE_TIME);
            Algorithm algorithm = Algorithm.HMAC256(secret);
            // 附带username信息
            return JWT.create()
                    .withClaim(SystemConstant.JWT_USERID, String.valueOf(userId))
                    .withClaim(SystemConstant.PLATFORM, platform.name())
                    .withClaim(SystemConstant.DEVICE_ID, deviceId)
                    .withClaim(SystemConstant.JWT_CURRENT_TIME, String.valueOf(start))
                    .withExpiresAt(date)
                    .sign(algorithm);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * 获取jwt失效时间
     *
     * @param token
     * @return
     */
    public static Date getExpirationDateFromToken(String token) {
        try {
            DecodedJWT jwt = JWT.decode(token);
            return jwt.getExpiresAt();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
}
